# Imports from Django.
from django import forms


# Imports from assignment-desk.
from assignment_desk.models import Assignment
from assignment_desk.models import Week


EXTRA_INFORMATION_HELP_TEXT = """
<strong>Optional.</strong> Use this field if you want to include a note
(with extra explanations, descriptions, etc.) that will be displayed
alongside this assignment list.
"""


class WeekCreationForm(forms.ModelForm):
    class Meta:
        model = Week

        fields = ['beginning_date', 'role_type', 'extra_information']
        labels = {'role_type': 'Type',}
        help_texts = {
            'extra_information': EXTRA_INFORMATION_HELP_TEXT,
        }


class WeekEditingForm(forms.ModelForm):
    class Meta:
        model = Week

        fields = ['extra_information']
        help_texts = {
            'extra_information': EXTRA_INFORMATION_HELP_TEXT,
        }


class AssignmentForm(forms.ModelForm):
    staffer = forms.CharField(widget=forms.HiddenInput())

    class Meta:
        model = Assignment
        fields = [
            'staffer',
            # 'notes',
        ]

InlineAssignmentFormset = forms.inlineformset_factory(
    Week,
    Assignment,
    form=AssignmentForm,
    extra=0,
    can_delete=False,
)



# Imports from Django.  # NOQA
from django.http import HttpResponseRedirect


class InlineFormsetMixin(object):
    form_types = ['form', 'inline_formset']

    def get_form_types(self):
        return self.form_types

    def get_inline_formset_kwargs(self):
        form_kwargs = {'instance': self.object}

        if self.request.method in ('POST', 'PUT',):
            form_kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })

        return form_kwargs

    def get_context_data(self, **kwargs):
        context = super(InlineFormsetMixin, self).get_context_data(**kwargs)

        # Only add the inline_formset to the context if it doesn't exist
        if 'inline_formset' not in kwargs:
            context['inline_formset'] = self.inline_formset_class(
                **self.get_inline_formset_kwargs()
            )

        # Check the main form and all forms in the inline formset for errors.
        # If any of these forms have a non-zero number of errors, set the
        # 'has_errors' key in the view context to True.

        # This logic exists because django-bootstrap3's built-in formset-error
        # renderer does not properly trace through formsets to display errors.
        if 'has_errors' not in kwargs:
            has_errors = max([
                len([
                    _ for _
                    in context[form_type].errors
                    if len(_) > 0
                ])
                for form_type in self.get_form_types()
            ]) > 0

            context.update(
                dict(
                    has_errors=has_errors
                )
            )

        return context

    def form_valid(self, form):
        """Override the form validator to handle"""
        context = self.get_context_data()
        inline_formset = context['inline_formset']

        if inline_formset.is_valid() and form.is_valid():
            # Save the ModelForm object, then move on to the inline formset
            self.object = form.save()
            inline_formset.instance = self.object

            # If the inline supports ordering and our view specifies an order
            # field, store the order on the model
            if inline_formset.can_order and hasattr(self, 'order_field'):
                order_field = getattr(self, 'order_field')
                for form in inline_formset.ordered_forms:
                    setattr(
                        form.instance,
                        order_field,
                        form.cleaned_data['ORDER']
                    )

            inline_formset.save()
            return HttpResponseRedirect(self.get_success_url())
        else:
            return self.render_to_response(self.get_context_data(
                form=form,
                inline_formset=inline_formset
            ))



# Imports from Django.
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponse
from django.views.generic import DetailView
from django.views.generic.edit import CreateView
from django.views.generic.edit import UpdateView
from django.views.generic.list import ListView


# Imports from assignment-desk.
from assignment_desk.forms import InlineAssignmentFormset
from assignment_desk.forms import WeekCreationForm
from assignment_desk.forms import WeekEditingForm
from assignment_desk.mixins import InlineFormsetMixin
from assignment_desk.models import Assignment
from assignment_desk.models import Role
from assignment_desk.models import Week


DAYS_IN_WEEK = [
    'monday',
    'tuesday',
    'wednesday',
    'thursday',
    'friday',
    'saturday',
    'sunday',
]


def index_view(request):
    return HttpResponse('Hello, world. You\'re at the assignment-desk index.')


class WeekListView(LoginRequiredMixin, ListView):
    queryset = Week.objects.all()
    template_name = 'assignment_desk/weeks/list.html'


class WeekCreateView(LoginRequiredMixin, CreateView):
    model = Week
    form_class = WeekCreationForm
    template_name = 'assignment_desk/weeks/create_form.html'


class WeekEditView(LoginRequiredMixin, InlineFormsetMixin, UpdateView):
    model = Week
    template_name = 'assignment_desk/weeks/edit_form.html'

    form_class = WeekEditingForm
    inline_formset_class = InlineAssignmentFormset

    def get_queryset(self):
        return Week.objects.prefetch_related(
            'assignments',
            'assignments__staffer',
            'assignments__role',
            'assignments__role__type',
        )

    # def get_initial(self):
    #     initial_data = super(WeekEditView, self).get_initial()

    #     # print(initial_data.assignments.values_list('id', flat=True))
    #     print(initial_data)

    #     initial_assignments = []

    #     initial_assignments.append(
    #         Assignment(
    #             week,
    #             # staffer,
    #             role=Role.objects.get(id=1),
    #             # day=
    #         )
    #     )

    #     initial_data['assignments'] = initial_assignments

    #     print()

    #     return initial_data

    # def get_context_data(self, **kwargs):
    #     pass


class WeekDetailView(LoginRequiredMixin, DetailView):
    queryset = Week.objects.all().prefetch_related(
        'assignments',
        'assignments__role',
        'assignments__staffer',
    )
    template_name = 'assignment_desk/weeks/detail.html'

    def get_context_data(self, **kwargs):
        context = super(WeekDetailView, self).get_context_data(**kwargs)

        grouped_assignments = {}

        for assignment in self.object.assignments.all():
            if assignment.role.name not in grouped_assignments:
                grouped_assignments[assignment.role.name] = {}

            day_name = assignment.day.strftime('%A').lower()

            grouped_assignments[
                assignment.role.name
            ][day_name] = assignment.staffer

        roles_for_type = Role.objects.filter(
            type_id=self.object.role_type.pk
        ).order_by('priority')

        context['available_roles'] = roles_for_type

        context['roles_with_assignments'] = {
            role.slug: {
                (day): (
                    grouped_assignments[role.name][day]
                    if role.name in grouped_assignments
                    and day in grouped_assignments[role.name]
                    else None
                ) for day in DAYS_IN_WEEK
            }
            for role in roles_for_type
        }

        # print(context['roles_with_assignments']['230-meeting']['friday'].name)

        return context



# Imports from Django.  # NOQA
from django.forms import CheckboxInput
# from django.forms import CheckboxSelectMultiple
# from django.forms import ClearableFileInput
from django.forms import DateInput
from django.forms import EmailInput
# from django.forms import FileInput
# from django.forms import MultiWidget
from django.forms import NumberInput
# from django.forms import RadioSelect
from django.forms import Select
from django.forms import Textarea
from django.forms import TextInput
from django.forms import URLInput


# Imports from other dependencies.
from bootstrap3.forms import render_label
from bootstrap3.renderers import FieldRenderer
from bootstrap3.text import text_value
from bootstrap3.utils import add_css_class


WIDGETS_WITH_UNDERLINE = (
    TextInput,
    DateInput,
    NumberInput,
    EmailInput,
    URLInput,
    Textarea,
)

WIDGETS_AFTER_LABEL = (
    Select,
)


class ImmaterialFieldRenderer(FieldRenderer):
    def post_widget_render(self, html):
        """Override adds helper class to checkboxes."""
        if isinstance(self.widget, CheckboxInput):
            return self.put_inside_label(
                '{}{}'.format(
                    html,
                    '<span class="ripple"></span><i class="helper"></i>'
                )
            )
        else:
            return super(ImmaterialFieldRenderer, self).post_widget_render(
                html
            )

    def add_label(self, html):
        """Override adds label _after_ input, & sometimes bar class."""
        label = self.get_label()
        if label:
            if isinstance(self.widget, WIDGETS_WITH_UNDERLINE):
                html = '{}{}{}'.format(
                    html,
                    render_label(
                        label,
                        label_for=self.field.id_for_label,
                        label_class=self.get_label_class()
                    ),
                    '<i class="bar"></i>'
                )
            elif isinstance(self.widget, WIDGETS_AFTER_LABEL):
                html = '{}{}'.format(
                    html,
                    render_label(
                        label,
                        label_for=self.field.id_for_label,
                        label_class=self.get_label_class()
                    )
                )
            elif isinstance(self.widget, Textarea):
                html = '{}{}'.format(
                    html,
                    render_label(
                        label,
                        label_for=self.field.id_for_label,
                        label_class=self.get_label_class()
                    )
                )
            else:
                html = '{}{}'.format(
                    render_label(
                        label,
                        label_for=self.field.id_for_label,
                        label_class=self.get_label_class()
                    ),
                    html
                )
        return html

    def get_form_group_class(self):
        """Override adds 'group_<field_name>' class to each group."""
        form_group_class = super(
            ImmaterialFieldRenderer,
            self
        ).get_form_group_class()

        named_group_class = 'group_{}'.format(self.field.name)

        form_group_class = add_css_class(form_group_class, named_group_class)

        return form_group_class

    def _render(self):
        """Overrides for root renderer.

        Adds helptext (append_to_field) later than usual, and adds class
        'select' when rendering a select box.
        """
        # See if we're not excluded
        if self.field.name in self.exclude.replace(' ', '').split(','):
            return ''
        # Hidden input requires no special treatment
        if self.field.is_hidden:
            return text_value(self.field)

        # Render the widget
        self.add_widget_attrs()
        if (isinstance(self.widget, Select)):
            new_classes = add_css_class(self.widget.attrs['class'], 'select')
            self.widget.attrs['class'] = new_classes

        if isinstance(self.widget, Textarea):
            self.widget.attrs['autocomplete'] = 'nope'
            # self.widget.attrs['autocorrect'] = 'off'
            # self.widget.attrs['autocapitalize'] = 'off'
            # self.widget.attrs['spellcheck'] = 'false'

            self.widget.attrs['rows'] = 6
            print(self.widget.attrs)

        html = self.field.as_widget(attrs=self.widget.attrs)

        self.restore_widget_attrs()

        # Start post render
        html = self.post_widget_render(html)

        html = self.wrap_widget(html)

        html = self.make_input_group(html)

        html = self.wrap_field(html)
        html = self.add_label(html)
        html = self.append_to_field(html)
        html = self.wrap_label_and_field(html)
        return html



# Imports from python.  # NOQA
from __future__ import unicode_literals


# Imports from Django.
from django.contrib import admin


# Imports from assignment-desk.
from assignment_desk.models import Assignment
from assignment_desk.models import Role
from assignment_desk.models import RoleType
from assignment_desk.models import Week


@admin.register(RoleType)
class RoleTypeAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('name', 'priority',),
        }),
        ("Don't touch unlesss you know what you're doing", {
            'classes': ('collapse',),
            'fields': ('slug',),
        }),
    )
    prepopulated_fields = {'slug': ('name',), }


@admin.register(Role)
class RoleAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('name', ('type', 'priority',),),
        }),
        ('More information', {
            'fields': ('color', 'description',),
        }),
        ("Don't touch unlesss you know what you're doing", {
            'classes': ('collapse',),
            'fields': ('slug',),
        }),
    )
    prepopulated_fields = {'slug': ('name',), }


class AssignmentInline(admin.TabularInline):
    model = Assignment


@admin.register(Week)
class WeekAdmin(admin.ModelAdmin):
    fields = (
        ('beginning_date', 'ending_date',),
        'role_type',
        'extra_information',
    )
    inlines = [AssignmentInline]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-06-14 20:26
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('assignment_desk', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='week',
            name='role_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='assignment_desk.RoleType'),
        ),
    ]



# Imports from python.  # NOQA
from __future__ import unicode_literals

# Imports from Django.
from django.db import models
from django.utils.dateformat import format as df


# Imports from other dependencies.
from colorfield.fields import ColorField
from editorial_staff.models import Staffer


# Imports from assignment-desk.
from assignment_desk.utils import get_latest_monday
from assignment_desk.utils import get_next_sunday


class RoleType(models.Model):
    name = models.CharField(max_length=50)
    slug = models.SlugField()

    priority = models.PositiveSmallIntegerField(default=10)

    class Meta:  # NOQA
        ordering = ['priority', 'name']

    def __str__(self):
        return self.name


class Role(models.Model):
    name = models.CharField(max_length=50)
    slug = models.SlugField()
    description = models.TextField(blank=True, null=True)
    color = ColorField(default='#0185D3')

    type = models.ForeignKey(RoleType, related_name='roles')

    priority = models.PositiveSmallIntegerField(default=10)

    class Meta:  # NOQA
        ordering = ['priority', 'name']

    def __str__(self):
        return '{} ({})'.format(self.name, self.type.name)


class Week(models.Model):
    beginning_date = models.DateField(default=get_latest_monday)
    ending_date = models.DateField(default=get_next_sunday)

    role_type = models.ForeignKey(
        RoleType,
        related_name='weeks',
        blank=True,
        null=True
    )

    assigned_staffers = models.ManyToManyField(
        Staffer,
        through='Assignment'
    )

    extra_information = models.TextField(blank=True, null=True)

    def __str__(self):
        return 'Week of {}'.format(df(self.beginning_date, 'N j, Y'))


class Assignment(models.Model):
    week = models.ForeignKey(
        Week,
        related_name='assignments',
        on_delete=models.CASCADE
    )
    staffer = models.ForeignKey(
        Staffer,
        related_name='assignments',
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )

    role = models.ForeignKey(
        Role,
        related_name='assignments',
        on_delete=models.CASCADE
    )
    day = models.DateField()
    notes = models.TextField(blank=True, null=True)

    class Meta:  # NOQA
        ordering = ['role', 'day']

    def __str__(self):
        return '{} — {} for {} ({})'.format(
            self.staffer,
            self.role.name,
            self.role.type.name,
            df(self.day, 'N j, Y')
        )



# Imports from Django.  # NOQA
from django.conf.urls import include
from django.conf.urls import url


# Imports from assignment-desk.
from assignment_desk.views import index_view
from assignment_desk.views import WeekCreateView
from assignment_desk.views import WeekDetailView
from assignment_desk.views import WeekEditView
from assignment_desk.views import WeekListView


app_name = 'assignment-desk'


urlpatterns = [
    url('^$', index_view, name='index'),
    url('^weeks/', include([
        url('^$', WeekListView.as_view(),
            name='week-list'),
        url('^create/$', WeekCreateView.as_view(),
            name='week-create'),
        url('^(?P<pk>[0-9]+)/', include([
            url('^$', WeekDetailView.as_view(),
                name='week-detail'),
            url('^edit/$', WeekEditView.as_view(),
                name='week-edit'),
            # url('^delete/$', WeekDeleteView.as_view(),
            #     name='week-delete'),
        ])),
    ])),
]
